package bildbearbeitung;

import mediacomputation.Picture;
import mediacomputation.Pixel;

/**
 * Kantendetektion mit Laplace Filter
 * http://de.wikipedia.org/wiki/Laplace-Filter
 * 
 * Implementation dazu bernommen von
 * http://www.cs.princeton.edu/introcs/31datatype/LaplaceFilter.java.html
 * 
 * Implementation ist einfach verstndlich geschrieben, aber nicht optimiert in
 * Bezug auf Aufhrungsgeschwindigkeit oder Speicherverbrauch.
 */
public class BildKantenEntdecken implements BildBearbeitungInterface {

	final double[][] FILTER = { { -1, -1, -1 }, { -1, 8, -1 }, { -1, -1, -1 } };

	@Override
	public Picture bearbeite(Picture orginalBild) {
		int breite = orginalBild.getWidth();
		int hoehe = orginalBild.getHeight();

		Picture neuesBild = new Picture(breite, hoehe);

		for (int y = 1; y < hoehe - 1; y++) {
			for (int x = 1; x < breite - 1; x++) {
				Pixel orginalPixel = orginalBild.getPixel(x, y);
				Pixel neuerPixel = neuesBild.getPixel(x, y);

				bearbeitePixel(orginalBild, orginalPixel, neuerPixel);
			}
		}

		return neuesBild;
	}

	void bearbeitePixel(Picture orginalBild, Pixel orginalPixel,
			Pixel neuerPixel) {

		int x = orginalPixel.getX();
		int y = orginalPixel.getY();

		Pixel pixelLinksOben = orginalBild.getPixel(x - 1, y - 1);
		Pixel pixelLinks = orginalBild.getPixel(x - 1, y);
		Pixel pixelLinksUnten = orginalBild.getPixel(x - 1, y + 1);
		Pixel pixelOben = orginalBild.getPixel(x, y - 1);
		Pixel pixel = orginalBild.getPixel(x, y);
		Pixel pixelUnten = orginalBild.getPixel(x, y + 1);
		Pixel pixelRechtsOben = orginalBild.getPixel(x + 1, y - 1);
		Pixel pixelRechts = orginalBild.getPixel(x + 1, y);
		Pixel pixelRechtsUnten = orginalBild.getPixel(x + 1, y + 1);

		double red = FILTER[0][0] * pixelLinksOben.getRed();
		red += FILTER[1][0] * pixelLinks.getRed();
		red += FILTER[2][0] * pixelLinksUnten.getRed();
		red += FILTER[0][1] * pixelOben.getRed();
		red += FILTER[1][1] * pixel.getRed();
		red += FILTER[2][1] * pixelUnten.getRed();
		red += FILTER[0][2] * pixelRechtsOben.getRed();
		red += FILTER[1][2] * pixelRechts.getRed();
		red += FILTER[2][2] * pixelRechtsUnten.getRed();

		double green = FILTER[0][0] * pixelLinksOben.getGreen();
		green += FILTER[1][0] * pixelLinks.getGreen();
		green += FILTER[2][0] * pixelLinksUnten.getGreen();
		green += FILTER[0][1] * pixelOben.getGreen();
		green += FILTER[1][1] * pixel.getGreen();
		green += FILTER[2][1] * pixelUnten.getGreen();
		green += FILTER[0][2] * pixelRechtsOben.getGreen();
		green += FILTER[1][2] * pixelRechts.getGreen();
		green += FILTER[2][2] * pixelRechtsUnten.getGreen();

		double blue = FILTER[0][0] * pixelLinksOben.getBlue();
		blue += FILTER[1][0] * pixelLinks.getBlue();
		blue += FILTER[2][0] * pixelLinksUnten.getBlue();
		blue += FILTER[0][1] * pixelOben.getBlue();
		blue += FILTER[1][1] * pixel.getBlue();
		blue += FILTER[2][1] * pixelUnten.getBlue();
		blue += FILTER[0][2] * pixelRechtsOben.getBlue();
		blue += FILTER[1][2] * pixelRechts.getBlue();
		blue += FILTER[2][2] * pixelRechtsUnten.getBlue();

		red = Math.min(255, Math.max(0, red));
		green = Math.min(255, Math.max(0, green));
		blue = Math.min(255, Math.max(0, blue));

		neuerPixel.setRed((int) red);
		neuerPixel.setGreen((int) green);
		neuerPixel.setBlue((int) blue);
	}

}
